package examples.visitor.java;

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the design patterns project at UBC
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http:*www.mozilla.org/MPL/ or http:*aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ca.ubc.cs.spl.pattern.
 *
 * Contributor(s):   
 */

/**
 * Implements a concrete visitor that collects the sum of all leaf values in 
 * the tree.
 *
 * @author  Jan Hannemann
 * @author  Gregor Kiczales
 * @version 1.0, 06/13/02
 * 
 * @see Node
 * @see Leaf
 * @see RegularNode
 */

public class SumVisitor implements NodeVisitor {
    
    /**
     * the colleced sum of leaf values
     */
    
	protected int sum = 0;
	 
    /**
     * Visits a RegularNode, which is a non-terminal binary tree node.
     *
     * @param node the regular node
     */
     
	public void visitRegularNode(Node node) {
		RegularNode rnode = (RegularNode) node; 
		rnode.left.accept(this);
		rnode.right.accept(this);
	}
		
    /**
     * Visits a Leaf, which is a terminal tree node.
     *
     * @param node the leaf
     */
     
	public void visitLeaf(Node node) {
		Leaf leaf = (Leaf) node;
		sum += leaf.getValue();
	}
	
	/** 
	 * Returns the result of the visitor's operation
	 *
	 * @returns a string describing the result of this visitor's operation.
	 */
	  
	public String report() {
		return ">>> SumVisitor collected a sum of "+sum;
	}
}